/*
 * hited.cc
 *
 *  Created on: June 1, 2012
 *      Author: jhuang
 * 
 * Install the core libraries:
 * :~/RNABarrier-distribution/trunk$ sudo apt-get install libboost-all-dev
 */

#include <map>
#include <string>
#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <iterator>
#include <vector>
#include <set>
#include <limits>
#include <cassert>

#include "comp/DebugTools.hpp"
#include "comp/Hishape.hpp"
#include "comp/HelixIndex.hpp"
#include "comp/ShapiroAligner.hpp"
#include "comp/NASecondaryStructure.hpp"
#include "comp/NASSDotBracketLoader.hpp"
#include "comp/TreeNode.hpp"
#include "comp/HelixIndex.hpp"

#include <boost/lexical_cast.hpp>
#include <boost/format.hpp>
#include <boost/program_options.hpp>
#include <boost/algorithm/string/replace.hpp>

namespace po = boost::program_options;

/*
  extent the main.cc with a new option 'P' for probability calculation
*/
// hishape abstraction type
const unsigned int HISHAPE_B                = 1;
const unsigned int HISHAPE_M                = 2;
const unsigned int HISHAPE_H_PLUS           = 3;
const unsigned int HISHAPE_H                = 4;


/* Auxiliary functions for checking input for validity. */

/* Function used to check that 'opt1' and 'opt2' are not specified
   at the same time. */
void conflicting_options(const po::variables_map& vm,
                         const char* opt1, const char* opt2)
{
    if (vm.count(opt1) && !vm[opt1].defaulted()
        && vm.count(opt2) && !vm[opt2].defaulted())
        throw std::logic_error(std::string("Conflicting options '")
                          + opt1 + "' and '" + opt2 + "'.");
}

/* Function used to check that of 'for_what' is specified, then
   'required_option' is specified too. */
void option_dependency(const po::variables_map& vm,
                        const char* for_what, const char* required_option)
{
    if (vm.count(for_what) && !vm[for_what].defaulted())
        if (vm.count(required_option) == 0 || vm[required_option].defaulted())
            throw std::logic_error(std::string("Option '") + for_what
                              + "' requires option '" + required_option + "'.");
}


void tokenize(const std::string& str,
                      std::vector<std::string>& tokens,
                      const std::string& delimiters = ",")
{
    // Skip delimiters at beginning.
    std::string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    std::string::size_type pos     = str.find_first_of(delimiters, lastPos);

    while (std::string::npos != pos || std::string::npos != lastPos)
    {
        // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}


// ################ functions for hishape comparison #################
// the third parameter of from_string() should be 
// one of std::hex, std::dec or std::oct
template <class T>
bool from_string(T& t, 
                 const std::string& s, 
                 std::ios_base& (*f)(std::ios_base&))
{
  std::istringstream iss(s);
  return !(iss >> f >> t).fail();
}

typedef val::biocpp::tools::CTreeNode<hishape::comp::CHelixIndex> CHelixIndexTreeNode;
//! A tree node that contains a pair of nucleotides.
CHelixIndexTreeNode::CTreeNodePointer HishapeToTree(std::string &strHishape)
{
	strHishape = strHishape.substr(1,strHishape.size()-2);
	//std::cout << "processed strHishape=" << strHishape << std::endl;
	

	// ###############################
	// #### prepare mmBracketPair ####
	std::multimap<int,int> mmBracketPair;
	std::multimap<int,int>::iterator itBracketPair;
	
	std::vector<std::string> helixIndices;
	tokenize(strHishape, helixIndices);  
	std::stack<int> helixIndexStack;
	int iHishapeLength = (signed)helixIndices.size();
	//BUG_TRACK  std::cout << iHishapeLength << std::endl;
	int i = 0;
	while (i < iHishapeLength)
	{
		if ("(" == helixIndices[i])
		{
			helixIndexStack.push(i++);
		}
		else if (")" == helixIndices[i])
		{
			if (true == helixIndexStack.empty())
			{
				throw std::invalid_argument("SNASSDotBracketLoader: The given sequence does not contain matching brackets!");
			}
			mmBracketPair.insert(std::pair<int,int>(helixIndexStack.top()-1,i));  // for m ... )
			//mmBracketPair.insert(std::pair<int,int>(helixIndexStack.top(),i));    // for ( ... )
			helixIndexStack.pop();
			i++;
		}
		else
		{
		        i++;
			//mmBracketPair.insert(std::pair<int,int>(i++,-1));
		}
	}    
	std::vector<std::string>::iterator it;
	for ( it=helixIndices.begin() ; it != helixIndices.end(); it++ )
	{
	    std::string currentHi = *it;
	    if ( (currentHi[currentHi.size()-1] != 'b') && (currentHi[currentHi.size()-1] != 'i') && \
		 (currentHi[currentHi.size()-1] != 'm') && (currentHi != "_") && (currentHi != "(") && (currentHi != ")") )  // in case h
	    {
	        //BUG_TRACK  std::cout << " " << *it;
                (*it) = (*it) + "h";
	    }
	}
	    


	// show content: 算出所有的 bracket对的位置
	//BUG_TRACK  for ( itBracketPair=mmBracketPair.begin() ; itBracketPair != mmBracketPair.end(); itBracketPair++ )
	//BUG_TRACK      std::cout << (*itBracketPair).first << " => " << (*itBracketPair).second << std::endl;
	



	
	//CHelixIndexTreeNode::CTreeNodePointer ptNode = NULL;
	//std::stack<std::pair<int, CHelixIndexTreeNode::CTreeNodePointer> > tStack;
	//int iHishapeLength = tHishape.GetLength();
	//int iIsolatedIndex = 0;
	
	
	

	// node root
	hishape::comp::CHelixIndex tHelixIndex(0.0f, 'N');
	CHelixIndexTreeNode::CTreeNodePointer rootNode = new CHelixIndexTreeNode(tHelixIndex);
	CHelixIndexTreeNode::CTreeNodePointer actualNode = rootNode;
	
	std::deque<int> mlPositionDeque;
	std::deque<CHelixIndexTreeNode::CTreeNodePointer> mlPointerDeque;
	
	// add the first level
	for (i = 0; i < iHishapeLength; i++)
	{
	    //std::cout << helixIndices[i] << std::endl;
	    std::string currentHi = helixIndices[i];
	    //std::cout << currentHi.substr (0,currentHi.size()-1) << std::endl;
	    char lastChar = currentHi.at(currentHi.size()-1);
	    float f;
	    if (lastChar == 'b' || lastChar == 'i')
	    {
	        //assert (from_string<float>(f, currentHi.substr (0,currentHi.size()-1), std::dec));
	        if(from_string<float>(f, currentHi.substr (0,currentHi.size()-1), std::dec))
                {
	            hishape::comp::CHelixIndex tHelixIndex(f, lastChar);
	            actualNode = actualNode->AddChild(tHelixIndex);
		}
		else
		{   
		    throw std::invalid_argument("Error occurs during transformation from string to float!");
		}
	    }
	    else if (lastChar == 'h')
	    {
	        if(from_string<float>(f, currentHi.substr (0,currentHi.size()-1), std::dec))
                {
	            hishape::comp::CHelixIndex tHelixIndex(f, lastChar);
	            actualNode->AddChild(tHelixIndex);
		}
		else
		{
		    throw std::invalid_argument("Error occurs during transformation from string to float!");
		}
		actualNode = rootNode;
	    }
	    else if (lastChar == 'm')
	    {
	        if(from_string<float>(f, currentHi.substr (0,currentHi.size()-1), std::dec))
                {
	            hishape::comp::CHelixIndex tHelixIndex(f, lastChar);
	            actualNode = actualNode->AddChild(tHelixIndex);
		}
		else
		{
		    throw std::invalid_argument("Error occurs during transformation from string to float!");
		}
		mlPositionDeque.push_back(i);
		mlPointerDeque.push_back(actualNode);
		actualNode = rootNode;
		i = mmBracketPair.find(i)->second;
	    }
	    
	}
	
	/*
	int mlPosition = mlPositionDeque.front();
	mlPositionDeque.pop_front();
	std::cout << mlPosition << std::endl;
	CHelixIndexTreeNode::CTreeNodePointer mlPointer = mlPointerDeque.front();
	mlPointerDeque.pop_front();
	hishape::comp::CHelixIndex tempHelixIndex(100.0f, 'z');
	mlPointer->AddChild(tempHelixIndex);
	*/

	// for further recursive calculation
	while (false == mlPositionDeque.empty())
	{
	    int mlPosition = mlPositionDeque.front();
	    mlPositionDeque.pop_front();
	    CHelixIndexTreeNode::CTreeNodePointer localRootNode = mlPointerDeque.front();
	    actualNode = localRootNode;
	    mlPointerDeque.pop_front();
	    int start = mlPosition + 1;                          // position of bracket open
	    int end = mmBracketPair.find(mlPosition)->second;    // position of bracket close
	    
	    for (i = (start+1); i < end; i++)
	    {
		//std::cout << helixIndices[i] << std::endl;
		std::string currentHi = helixIndices[i];
		//std::cout << currentHi.substr (0,currentHi.size()-1) << std::endl;
		char lastChar = currentHi.at(currentHi.size()-1);
		float f;
		if (lastChar == 'b' || lastChar == 'i')
		{
		    //assert (from_string<float>(f, currentHi.substr (0,currentHi.size()-1), std::dec));
		    if(from_string<float>(f, currentHi.substr (0,currentHi.size()-1), std::dec))
		    {
			hishape::comp::CHelixIndex tHelixIndex(f, lastChar);
			actualNode = actualNode->AddChild(tHelixIndex);
		    }
		    else
		    {   
			throw std::invalid_argument("Error occurs during transformation from string to float!");
		    }
		}
		else if (lastChar == 'h')
		{
		    if(from_string<float>(f, currentHi.substr (0,currentHi.size()-1), std::dec))
		    {
			hishape::comp::CHelixIndex tHelixIndex(f, lastChar);
			actualNode->AddChild(tHelixIndex);
		    }
		    else
		    {
			throw std::invalid_argument("Error occurs during transformation from string to float!");
		    }
		    actualNode = localRootNode;
		}
		else if (lastChar == 'm')
		{
		    if(from_string<float>(f, currentHi.substr (0,currentHi.size()-1), std::dec))
		    {
			hishape::comp::CHelixIndex tHelixIndex(f, lastChar);
			actualNode = actualNode->AddChild(tHelixIndex);
		    }
		    else
		    {
			throw std::invalid_argument("Error occurs during transformation from string to float!");
		    }
		    mlPositionDeque.push_back(i);
		    mlPointerDeque.push_back(actualNode);
		    actualNode = localRootNode;
		    i = mmBracketPair.find(i)->second;
		}
	    }
	}
	
	
	CHelixIndexTreeNode::CTreeNodePointerList tSuffixNodesList1 = rootNode->GetSuffixList();
	////std::cout << "Suffix order for tree 1:" << std::endl;
	std::string strSuffix1("[");
	for (CHelixIndexTreeNode::CTreeNodePointerList::iterator tSuffixIter1 = tSuffixNodesList1.begin(); tSuffixIter1 != tSuffixNodesList1.end(); tSuffixIter1++)
	{
		strSuffix1 += (*tSuffixIter1)->GetObject().ToString() /*GetPairBaseCodes()*/ + ", ";
	}
	strSuffix1.resize((signed)strSuffix1.size() - 2);
	strSuffix1 += "]";
	//BUG_TRACK  std::cout << strSuffix1 << std::endl;

	return rootNode;
	
}


std::string PrintStemLoop(const val::biocpp::CNASecondaryStructure::SSubstructureData &tRetData, const val::biocpp::CNASecondaryStructure &tSecStructure, unsigned int hishape_type)
{
        std::ostringstream outs;
	int iPosition=tRetData.iStartIndex1;
	for ( ; tSecStructure.GetNucleotideConfig(iPosition+1)!=val::biocpp::na_secondary_structure::I_HAIRPIN_LOOP; iPosition++)
	{       
                // because the hairpin.size() >= 3, the position (iPosition+2) is always safe
	        //if (tSecStructure.GetNucleotideConfig(iPosition+2)==val::biocpp::na_secondary_structure::I_HAIRPIN_LOOP)
	        //{
		//    break;
		//}
		if ( tSecStructure.IsPaired(iPosition) && (!tSecStructure.IsPaired(iPosition+1) || !tSecStructure.IsPaired(tSecStructure.GetPairedBaseIndex(iPosition)-1)) ) 
		{
		      float helixIndex = (iPosition + tSecStructure.GetPairedBaseIndex(iPosition))/2.0f;
		      int checkPosition = iPosition+1;
		      if (tSecStructure.IsPaired(checkPosition))
		      {
			  checkPosition = tSecStructure.GetPairedBaseIndex(iPosition)-1;
		      }
		      assert (!tSecStructure.IsPaired(checkPosition));

		      switch (tSecStructure.GetNucleotideConfig(checkPosition))
		      {
			    case val::biocpp::na_secondary_structure::I_EXTERNAL_BASE:
			    {
				    outs << "Error_I_EXTERNAL_BASE";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_STACKED_PAIR:
			    {
				    outs << "Error_I_STACKED_PAIR";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_BULGE:
			    {
			            if (hishape_type == HISHAPE_B)
				        outs << helixIndex << "b,";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_INTERNAL_LOOP:
			    {
			            if (hishape_type == HISHAPE_B)
				        outs << helixIndex << "i,";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_HAIRPIN_LOOP:
			    {
				    outs << "Error_I_HAIRPIN_LOOP";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_MULTIBRANCHED_LOOP:
			    {
				    outs << "Error_I_MULTIBRANCHED_LOOP";
				    break;
			    }
			    default:
			    {
				    outs << "Error_" << tSecStructure.GetNucleotideConfig(iPosition+1);
			    }
		      }
		}
	}
	assert (tSecStructure.IsPaired(iPosition));
	outs << (iPosition+tSecStructure.GetPairedBaseIndex(iPosition))/2.0f << ","; 
	return outs.str();
}


std::string PrintStem(const val::biocpp::CNASecondaryStructure::SSubstructureData &tRetData, const val::biocpp::CNASecondaryStructure &tSecStructure, unsigned int hishape_type)
{
        std::ostringstream outs;
	for (int iPosition=tRetData.iStartIndex1; iPosition<tRetData.iEndIndex1; iPosition++)
	{
		if ( tSecStructure.IsPaired(iPosition) && (!tSecStructure.IsPaired(iPosition+1) || !tSecStructure.IsPaired(tSecStructure.GetPairedBaseIndex(iPosition)-1)) ) 
		{
		      float helixIndex = (iPosition + tSecStructure.GetPairedBaseIndex(iPosition))/2.0f;
		      int checkPosition = iPosition+1;
		      if (tSecStructure.IsPaired(checkPosition))
		      {
			  checkPosition = tSecStructure.GetPairedBaseIndex(iPosition)-1;
		      }
		      assert (!tSecStructure.IsPaired(checkPosition));


		      switch (tSecStructure.GetNucleotideConfig(checkPosition))
		      {
			    case val::biocpp::na_secondary_structure::I_EXTERNAL_BASE:
			    {
				    outs << "Error_I_EXTERNAL_BASE";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_STACKED_PAIR:
			    {
				    outs << "Error_I_STACKED_PAIR";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_BULGE:
			    {
			            if (hishape_type == HISHAPE_B)
				        outs << helixIndex << "b,";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_INTERNAL_LOOP:
			    {
			            if (hishape_type == HISHAPE_B)
				        outs << helixIndex << "i,";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_HAIRPIN_LOOP:
			    {
				    outs << "Error_I_HAIRPIN_LOOP";
				    break;
			    }
			    case val::biocpp::na_secondary_structure::I_MULTIBRANCHED_LOOP:
			    {
				    outs << "Error_I_MULTIBRANCHED_LOOP";
				    break;
			    }
			    default:
			    {
				    outs << "Error_" << tSecStructure.GetNucleotideConfig(iPosition+1);
			    }
		      }
		}
	}
	if (hishape_type <= HISHAPE_M)
	{
	    outs << (tRetData.iEndIndex1+tRetData.iStartIndex2)/2.0f << 'm' << ",";
	}
	return outs.str();
}
	

typedef val::biocpp::tools::CTreeNode<hishape::comp::CHelixIndex> CHelixIndexTreeNode;
std::string PrintRecursive( const CHelixIndexTreeNode::CTreeNodePointer &treeNode, const val::biocpp::CNASecondaryStructure &tSecStructure, const std::multimap<int,int> &mmFirstPosJ, unsigned int hishape_type)
{
	CHelixIndexTreeNode::CTreeNodePointerList suffixList_Children = treeNode->GetDirectChildrenList();
	CHelixIndexTreeNode::CTreeNodePointerList::iterator tIter = suffixList_Children.begin();
        std::ostringstream outs;
	
	while (tIter != suffixList_Children.end())
        {
		
		int j = mmFirstPosJ.find((*tIter)->GetObject().GetHiNumber())->second;
		val::biocpp::CNASecondaryStructure::SSubstructureData tRetData = tSecStructure.GetStemOrStemLoopData(j);
		if (tRetData.iStructureNature == val::biocpp::na_secondary_structure::I_STEM_TYPE)  // knot m
		{
		    outs << PrintStem(tRetData, tSecStructure, hishape_type);
		    if (hishape_type <= HISHAPE_H_PLUS)    outs << "(,";
		    outs << PrintRecursive(*tIter, tSecStructure, mmFirstPosJ, hishape_type);
		    if (hishape_type <= HISHAPE_H_PLUS)    outs << "),";
		}   
		else    // leaf
		{
		    outs << PrintStemLoop(tRetData, tSecStructure, hishape_type);
		    outs << PrintRecursive(*tIter, tSecStructure, mmFirstPosJ, hishape_type);
		}

		tIter++;
	}
	return outs.str();
}

// transform secondary structure to hishape
std::string sSToHishape( const std::string &strStructure1, int hishape_type)
{
	val::biocpp::loaders::SNASSDotBracketLoader tDotBracketLoader;
	// TODO: use class CStem instead of class CHelixIndex in Tree
	// ## reuse following data structures
	std::multimap<int,int> mmFirstPosJ;
	std::multimap<int,int>::iterator itFirstPosJ;
	
	
	// ###################################
	// #### convert SS1 to Hishape1 ####
	// ###################################
	val::biocpp::CNASecondaryStructure tSecStructure1 = tDotBracketLoader(strStructure1);
	
	// #####################################################
	// #### get the tree structure of StemOrStemLoopData
	
	CHelixIndexTreeNode::CTreeNodePointer ptRoot1 = tSecStructure1.ParseStructureWithTree();
	CHelixIndexTreeNode::CTreeNodePointerList rootSuffix1 = ptRoot1->GetSuffixList();
	
	// #####################################################
	// #### prepare a map(firstPosition, j)
	int iSecondStructureStemsCount1 = tSecStructure1.GetStemsAndStemLoopsCount();
	val::biocpp::CNASecondaryStructure::SSubstructureData tRetData1;
	
	for (int j1=1; j1 <= iSecondStructureStemsCount1; ++j1)
	{
		tRetData1 = tSecStructure1.GetStemOrStemLoopData(j1);
		mmFirstPosJ.insert (std::pair<int,int>(tRetData1.iStartIndex1,j1));
	}
	
	// #####################################################
	// #### first round and further recursive
	std::ostringstream stmHishape;
	//if (hishape_type <= HISHAPE_H_PLUS)    std::cout << "(,";
	stmHishape << "[";
	CHelixIndexTreeNode::CTreeNodePointerList ptRootChildren1 = ptRoot1->GetDirectChildrenList();
	for (CHelixIndexTreeNode::CTreeNodePointerList::iterator tIter1 = ptRootChildren1.begin(); tIter1 != ptRootChildren1.end(); tIter1++)
	{
	    //if (hishape_type <= HISHAPE_H_PLUS)    std::cout << "(,";
	    int j1 = mmFirstPosJ.find((*tIter1)->GetObject().GetHiNumber())->second;
	    val::biocpp::CNASecondaryStructure::SSubstructureData tRetData1 = tSecStructure1.GetStemOrStemLoopData(j1);
	    if (tRetData1.iStructureNature == val::biocpp::na_secondary_structure::I_STEM_TYPE)  // knot m
	    {
		    stmHishape << PrintStem(tRetData1, tSecStructure1, hishape_type);
		    if (hishape_type <= HISHAPE_H_PLUS)    stmHishape << "(,";
		    stmHishape << PrintRecursive(*tIter1, tSecStructure1, mmFirstPosJ, hishape_type);
		    if (hishape_type <= HISHAPE_H_PLUS)    stmHishape << "),";
	    }   
	    else    // leaf
	    {
		stmHishape << PrintStemLoop(tRetData1, tSecStructure1, hishape_type);
		stmHishape << PrintRecursive(*tIter1, tSecStructure1, mmFirstPosJ, hishape_type);
	    }
	    //if (hishape_type <= HISHAPE_H_PLUS)    std::cout << "),";
	}
	//if (hishape_type <= HISHAPE_H_PLUS)    std::cout << "),";
	std::string strHishape = stmHishape.str();
	if (!strHishape.empty())
	{
	    strHishape = strHishape.substr(0,strHishape.size()-1);
	}
	
	strHishape = strHishape + "]";
	delete ptRoot1;
	return strHishape; 
}

int pairAlign(std::string strInput1, std::string strInput2, int hishape_type, float ratio)
{
	// Code for Trim trailing Spaces only
	size_t endpos = strInput1.find_last_not_of("\n"); // Find the first character position from reverse
	if ( std::string::npos != endpos )
	    strInput1 = strInput1.substr( 0, endpos+1 );
	endpos = strInput2.find_last_not_of("\n");
	if ( std::string::npos != endpos )
	    strInput2 = strInput2.substr( 0, endpos+1 );
	
	try
	{
		// #####################################################################
		// #### transfer a hishape type h_plus, m, b into the tree ####
		//BUG_TRACK  std::cout << "strInput1=" << strInput1 << std::endl;
		//BUG_TRACK  std::cout << "strInput2=" << strInput2 << std::endl;
		//BUG_TRACK  std::cout << "hishape_type=" << hishape_type << std::endl;
		if (hishape_type==HISHAPE_H_PLUS) {
		    boost::replace_all(strInput1, "(", "-1m,(");
		    boost::replace_all(strInput2, "(", "-1m,(");
		}
		//BUG_TRACK  std::cout << "strInput1=" << strInput1 << std::endl;
		//BUG_TRACK  std::cout << "strInput2=" << strInput2 << std::endl;
		CHelixIndexTreeNode::CTreeNodePointer ptTree1 = HishapeToTree(strInput1);
		CHelixIndexTreeNode::CTreeNodePointer ptTree2 = HishapeToTree(strInput2);
		
		if (ratio < 0.0f || ratio > 10000.0f) 
		{
		    std::cout << "The ratio value should be specified between 0.0 and 10000.0!" << std::endl;
		    return 1;
		}
		hishape::comp::algorithms::CShapiroAligner tShapiroAligner(ptTree1, ptTree2, ratio);
		hishape::comp::CScoreScheme tScoreScheme;
		tShapiroAligner.SetScoreScheme(tScoreScheme);
		tShapiroAligner.Align();
		// TODO: fitting the score calculation
		//tShapiroAligner.SetFirstTree(HishapeToTree("[27]"));
		//tShapiroAligner.SetSecondTree(HishapeToTree("[36.5m,(,27,38,)]"));
		//tShapiroAligner.Align(false,false);
		
		//BUG_TRACK  std::cout << "aligning finished!" << std::endl;
		std::cout << tShapiroAligner.GetScore() << std::endl;                     // delete tShapiroAligner?? 
		return 0;
	}
	catch (std::exception &ex)
	{
		std::cout << "ERROR: " << ex.what() << std::endl;
	}
	catch (...)
	{
		std::cout << "ERROR: unexpected exception!" << std::endl;
	}
}

// END FUNCTIONS
//******************************************************************************


int main(int argc, char **argv)
{  
  
    // ########################
    // ## options            ##
    // ########################
        /* options declaration */
    //bool window_mode = false;
    //#ifdef WINDOW_MODE
        //    bool window_mode = true;
    //#endif


    unsigned int hishape_type;
    float ratio = 5.0f;  // TODO: describe the ratio is correspondent to \lambda in the paper in option description
    unsigned int convert = 0;

    try {
      
        // idea is the option is not compulsory and it will be given a relexed code  
        //po::options_description desc("Allowed options", 120, 60);
        po::options_description desc("Allowed options");
	
	desc.add_options()
	("file,f", po::value< std::vector<std::string> >(), "Read sequence(s) from specified file")
	("type,t", po::value<unsigned int>(&hishape_type)->default_value(2), "Specify hishape abstract type")
        ("ratio,r", po::value<float>(&ratio)->implicit_value(5.0f), "Specify weight between the score from helix index distance and score from helix index type conversion")
	("convert,c", po::value<unsigned int>(&convert)->implicit_value(1), "Convert secondary structure to hishape")
	("help,h", "Produce help message")
	("version,v", "Show version");


        /*
         * it can add at most additional 2 records, that means,
         * it is still probable that the sum of the records >= 2
         *
         * ./main --help -f aaa dddd ccc -s bbb
         * will get the result
         * input files are: aaa
         * input sequences are: dddd ccc bbb
         * positional options are: dddd ccc bbb
         */
        po::positional_options_description p;
        p.add("file", -1);


        po::variables_map vm;
        po::store(po::command_line_parser(argc, argv).
                  options(desc).positional(p).run(), vm);
        //po::store(parse_command_line(argc, argv, desc), vm);
        po::notify(vm);

        // option exclusions and implications
	conflicting_options(vm, "help", "version");
	conflicting_options(vm, "file", "sequence");
	conflicting_options(vm, "align", "convert");


        //std::cout << "vm.size()=" << vm.size() << std::endl;
        if (vm.count("help") || vm.size()==1) {
            std::cout << "Usage: HiTed [<first hishape>] [<second hishape>] [options]\n";
	    std::cout << "Examples:" << std::endl;
	    std::cout << "  ./HiTed '[27]' '[36.5m,(,27,38,)]' -r 1" << std::endl;
	    std::cout << "  ./HiTed ../examples/riboswitches.fas -t 1 -r 1" << std::endl;  // compare the 1st and 2nd, 3rd and 4nd
	    std::cout << "  ./HiTed ../examples/riboswitches.fas -c" << std::endl;            // convert every record 
            std::cout << desc; //<< '\n';
            return 0;
        }

        if (vm.count("version")) {
            std::cout << "HiTed 2.0.1 (Aug. 14, 2012)\n";  // TODO: use the C++ buildin date function 
            return 0;
        }
        

       if (vm.count("file"))
       {
	    std::vector<std::string> files = vm["file"].as< std::vector<std::string> >();
            if (!files.empty()) {
	        // ##################
	        // ## convert mode ##
	        if (convert==1 && files.size()==1) 
		{
		  	if (hishape_type < 1 || hishape_type > 4) 
			{
				std::cout << "The hishape abstract type can only be specified between 1 and 4." << std::endl;  // For multiple hishape alignment, 
				return 1;
			}

			std::string line;
			std::fstream fiInputFile;
			if (false == fiInputFile.is_open())
			{       
					// get first input file
					fiInputFile.open(files[0].c_str(), std::ios::in);
					// checks if the file was opened
					if ((false == fiInputFile) || (false == fiInputFile.is_open()))
					{
						// not opened, throws an exception
						throw "ERROR: loading input file (\"" + files[0] + "\")!";
					}
					
					std::string header = "";
					std::string seq = "";
					std::string ss = "";
					int line_no = 0;
					while ( getline(fiInputFile, line) )
					{

						if ( (line_no%3 == 0) ) {
						    if (line[0] == '>')
						    {
							// get header only
							//header = fasta.substr(1);
							header = line;
						    }
						    else
						    {
							throw "ERROR: wrong faa-format in input file (\"" + files[0] + "\")!";
						    }
						}
						else 
						{
						    if (line_no%3 == 1)
							seq = line;
						    if (line_no%3 == 2)
							ss = line;
						}


						if (header != "" && seq != "" && ss != "") {
							std::cout << header << std::endl;
							std::cout << seq << std::endl;
							std::cout << ss << std::endl;
                                                        std::cout << sSToHishape(header+"\n"+seq+"\n"+ss+"\n", hishape_type) /*<< " (hishape type = " << hishape_type << ")" */<< std::endl;
							header = "";
							seq = "";
							ss = "";
						}
						line_no ++;
					}
			}
			else
			{
				std::cout << "ERROR: unable to open input file (\"" + files[0] + "\")!\n";
			}
			
			if (true == fiInputFile.is_open())
			{
				fiInputFile.close();
			}
		}
                else
	        // ##################
	        // #### align mode ##
		if (files.size()==2) 
		{   
		        std::string strInput1,strInput2;
		        if (!files[0].empty() && files[0].at(0) == '[' &&  !files[1].empty() && files[1].at(0) == '[')
			{
				strInput1 = files[0];
				strInput2 = files[1];
				pairAlign(strInput1, strInput2, hishape_type, ratio);
			}
		}
		else
		if (files.size()==1) 
		{
			std::string line;
			std::fstream fiInputFile;
			if (false == fiInputFile.is_open())
			{       
					// get first input file
					fiInputFile.open(files[0].c_str(), std::ios::in);
					// checks if the file was opened
					if ((false == fiInputFile) || (false == fiInputFile.is_open()))
					{
						// not opened, throws an exception
						throw "ERROR: loading input file (\"" + files[0] + "\")!";
					}

					std::string header1 = "";
					std::string seq1 = "";
					std::string ss1 = "";
					std::string header2 = "";
					std::string seq2 = "";
					std::string ss2 = "";
					int line_no = 0;
					while ( getline(fiInputFile, line) )
					{

						if ( (line_no%6 == 0) ) {
						    if (line[0] == '>')
						    {
							// get header only
							//header1 = line.substr(1);
							header1 = line;
						    }
						    else
						    {
							throw "ERROR: wrong faa-format in input file (\"" + files[0] + "\")!";
						    }
						}
						else if ( (line_no%6 == 3) ) {
						    if (line[0] == '>')
						    {
							// get header only
							//header2 = line.substr(1);
							header2 = line;
						    }
						    else
						    {
							throw "ERROR: wrong faa-format in input file (\"" + files[0] + "\")!";
						    }
						}
						else 
						{
						    if (line_no%6 == 1)
							seq1 = line;
						    if (line_no%6 == 2)
							ss1 = line;
						    if (line_no%6 == 4)
							seq2 = line;
						    if (line_no%6 == 5)
							ss2 = line;
						}


						if (header1 != "" && header2 != "" && seq1 != "" && seq2 != "" && ss1 != "" && ss2 != "") {
						        std::cout << "#### " << header1 + " vs. " + header2 + " ####" << std::endl;  
                                                        pairAlign(sSToHishape(header1+"\n"+seq1+"\n"+ss1+"\n", hishape_type), sSToHishape(header2+"\n"+seq2+"\n"+ss2+"\n", hishape_type), hishape_type, ratio);
							header1 = "";
							header2 = "";
							seq1 = "";
							seq2 = "";
							ss1 = "";
							ss2 = "";
						}
						line_no ++;
					}
			}
			else
			{
				std::cout << "ERROR: unable to open input file (\"" + files[0] + "\")!\n";
			}
			
			if (true == fiInputFile.is_open())
			{
				fiInputFile.close();
			}
		}
		else
		{
		    std::cout << "Status: files.size()=" << files.size() << std::endl;
		}
	    }
        }
    } catch (std::exception &e) {
        std::cerr << "Exception: " << e.what() << '\n';
        std::exit(1);
    }


  return 0;
}